import {
  ROOT,
  Fiber,
  EffectTag,
  platform,
  flatten,
  detectIsUndefined,
  TagVirtualNode,
  rootStore,
  wipRootStore,
  currentRootStore,
  nextUnitOfWorkStore,
  fiberMountStore,
  TaskPriority,
  createReplacer,
  isInsertionEffectsZone,
  isLayoutEffectsZone,
} from '@dark-engine/core';
import { createNativeElement, applyCommit, finishCommitWork } from '../dom';
import { detectIsPortal, unmountPortal } from '../portal';
import { scheduleCallback, shouldYeildToHost } from '../scheduler';
platform.createNativeElement = createNativeElement;
platform.requestAnimationFrame = requestAnimationFrame.bind(this);
platform.cancelAnimationFrame = cancelAnimationFrame.bind(this);
platform.scheduleCallback = scheduleCallback;
platform.shouldYeildToHost = shouldYeildToHost;
platform.applyCommit = applyCommit;
platform.finishCommitWork = finishCommitWork;
platform.detectIsPortal = detectIsPortal;
platform.unmountPortal = unmountPortal;
const roots = new Map();
function render(element, container) {
  if (!(container instanceof Element)) {
    throw new Error(`[Dark]: render receives only Element as container!`);
  }
  const isMounted = !detectIsUndefined(roots.get(container));
  let rootId = null;
  if (!isMounted) {
    rootId = roots.size;
    roots.set(container, rootId);
    container.innerHTML = '';
  } else {
    rootId = roots.get(container);
  }
  if (isInsertionEffectsZone.get(rootId)) return;
  const callback = () => {
    rootStore.set(rootId);
    const currentRoot = currentRootStore.get();
    const fiber = new Fiber().mutate({
      nativeElement: container,
      instance: new TagVirtualNode({
        name: ROOT,
        children: flatten([element || createReplacer()]),
      }),
      alternate: currentRoot,
      effectTag: isMounted ? EffectTag.UPDATE : EffectTag.CREATE,
    });
    currentRoot && (currentRoot.alternate = null);
    fiberMountStore.reset();
    wipRootStore.set(fiber);
    nextUnitOfWorkStore.set(fiber);
  };
  platform.scheduleCallback(callback, {
    priority: TaskPriority.NORMAL,
    forceSync: isLayoutEffectsZone.get(),
  });
}
export { render, roots };
//# sourceMappingURL=render.js.map
